# coding: utf-8

import base64
import os
from zipfile import ZipFile

import jinja2
import yaml
from faker import Factory
from flask import render_template
from flask import session
from flask.json import jsonify

from helpers import create_organization
from helpers import create_person
from helpers import param_parser
from helpers import send_request
from names import SYSTEMS
from objects import get_id
from objects import Context
from server import app

# CONFIG DATA ################################
opt = param_parser()
system = SYSTEMS[opt['system']]
conf_path = os.path.join(os.getenv('CONF_PATH'), 'conf.yaml')
config = yaml.load(open(conf_path))
SOURCE_SYSTEM_ID = config['SOURCE_SYSTEM'][system.upper()]['ID']
AUTH_KEY = config['SOURCE_SYSTEM'][system.upper()]['AUTH_KEY']
DESTINATION_SYSTEM_ID = config['DESTINATION_SYSTEM']['ID']
SESSION_ID = str(get_id()) + str(get_id())
URL = config['RECIPIENT']['URL']
is_debug = config['DEBUG']
is_send = opt['send'].lower()
CONTEXT = Context()
CONTEXT.run_info = {
    'system': opt['system'],
    'region': opt['region'],
    'city': opt['city'],
    'persons': opt['persons'],
    'organizations': opt['organizations']+1,
    'oktmo': opt['oktmo'],
    'is_send': is_send
}
##############################################


def main():

    # GENERATE DATA ##############################
    persons = []
    organizations = []
    fake = Factory.create('ru_RU')
    org_for_all_persons = create_organization(
        region=opt['region'],
        city=opt['city'],
        oktmo=opt['oktmo'],
        system=system,
        faker=fake
    )
    CONTEXT.organizations = [org_for_all_persons, ]
    for _ in xrange(opt['persons']):
        persons.append(create_person(
            fake,
            org_for_all_persons,
            system
        ))
    CONTEXT.persons = persons
    for _ in xrange(opt['organizations']):
        organizations.append(create_organization(
            region=opt['region'],
            city=opt['city'],
            oktmo=opt['oktmo'],
            system=system,
            faker=fake
        ))
    CONTEXT.organizations.extend(organizations)
    ##############################################

    # CREATE DATA PACKAGE ########################
    env = jinja2.Environment()
    temp_path = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'templates/'
    )
    env.loader = jinja2.FileSystemLoader(temp_path)
    TEMP_FILE_NAME = 'package_{}.xml'.format(system)
    PACKAGE_FILE_NAME = 'data.xml'
    temp = env.get_template(TEMP_FILE_NAME)
    with open(PACKAGE_FILE_NAME, 'w') as output:
        output.write(temp.render(
            persons=persons,
            organizations=organizations,
            source_system_id=SOURCE_SYSTEM_ID,
            auth_key=AUTH_KEY,
            destination_system_id=DESTINATION_SYSTEM_ID,
            session_id=SESSION_ID
        ).encode('utf-8'))
    ##############################################

    # CREATE DATA ARCHIVE AND ENCODING ###########
    PACKAGE_ARCHIVE_NAME = PACKAGE_FILE_NAME.replace('xml', 'zip')
    with ZipFile(PACKAGE_ARCHIVE_NAME, 'w') as archive:
        archive.write(PACKAGE_FILE_NAME)

    with open(PACKAGE_ARCHIVE_NAME, 'r') as archive:
        encode_archive = base64.b64encode(archive.read())
    ##############################################

    # CREATE QUERY FILE ##########################
    QUERY_FILE_NAME = 'query.xml'
    query_temp = env.get_template(QUERY_FILE_NAME)
    with open('query_data.xml', 'w') as output:
        output.write(query_temp.render(
            xml_package_name=PACKAGE_FILE_NAME,
            query_package_base64=encode_archive,
            auth_key=AUTH_KEY,
            source_system_id=SOURCE_SYSTEM_ID,
            destination_system_id=DESTINATION_SYSTEM_ID,
            session_id=SESSION_ID
        ).encode('utf-8'))
    ##############################################

    # SEND REQUEST ###############################
    if is_send == 'true':
        CONTEXT.run_info['send_result'] = send_request(URL)

    ##############################################

    # DELETE TEMP FILES ##########################
    if not is_debug:
        for f in (PACKAGE_ARCHIVE_NAME, PACKAGE_FILE_NAME):
            os.remove(f)
    ##############################################

    # VIEWS ######################################
    @app.route('/')
    def home():
        return render_template(
            'index.html',
            persons=CONTEXT.persons,
            organizations=CONTEXT.organizations,
            run_info=CONTEXT.run_info
        )

    @app.route('/send_data', methods=['POST'])
    def send_data():
        if (not session.get('was_send') or
                not CONTEXT.run_info.get('send_result')):
            result = send_request(URL)
            session['was_send'] = True
            CONTEXT.run_info['send_result'] = result
            CONTEXT.run_info['is_send'] = 'true'
        else:
            result = CONTEXT.run_info['send_result']
        return jsonify(result)
    ##############################################

    # RUN SERVER #################################
    app.run(host='0.0.0.0')
    ##############################################

if __name__ == '__main__':
    main()