# coding: utf-8
from datetime import datetime
from random import randint
from names import AdaptationProgram
from names import COLLEGE_SUBJECTS
from names import DifficultSituation
from names import DisabilityGroup
from names import HealthGroups
from names import KINDERGARTEN_NAMES
from names import LongTermTreatment
from names import PhysicalCultureGroup
from names import SCHOOL_SUBJECTS


def get_id():
    return datetime.now().strftime('%y%m%d%H%M%S%f')


class Organization(object):
    u"""
    Класс для формирования данных по организации.
    """
    def __init__(self, region, city, oktmo, system, factory):
        self.id = get_id()
        self.name = self._get_name(system, factory)
        self.region = region
        self.city = city
        self.oktmo = oktmo
        self.inn = self._get_inn()
        self.kpp = randint(9000000, 9999999)
        self.ogrn = self._get_ogrn()
        self.other_id = get_id()
        self.prof_id = get_id()
        self.act_cap = randint(500, 750)
        self.max_cap = randint(750, 1000)

    @staticmethod
    def _get_inn():
        factors = [2, 4, 10, 3, 5, 9, 4, 6, 8]
        numbers = [randint(1, 9) for _ in xrange(9)]
        str_num = [str(n) for n in numbers]
        numbers = sum([factors[i]*numbers[i] for i in xrange(len(numbers))])
        check_sum = str(numbers % 11) if numbers % 11 != 10 else '0'

        return ''.join([''.join(str_num), check_sum])

    @staticmethod
    def _get_ogrn():
        num = randint(9100000000, 9999999999)
        return ''.join([str((num*11)+3), '3'])
    
    @staticmethod
    def _get_name(system, factory):
        if system.lower() == 'dou':
            return KINDERGARTEN_NAMES[randint(0, len(KINDERGARTEN_NAMES)-1)]
        else:
            return factory.company().split(' ')[0].replace(',', '')


def mult(inn, factors):
    return [int(inn[i])*factors[i] for i in xrange(len(factors))]


class Person(object):
    u"""
    Класс для формирования данных по персоне.
    """
    def __init__(
            self, firstname, middlename, lastname,
            delegate_firstname, delegate_middlename,
            delegate_lastname, organization, system
    ):
        self.id = get_id()
        self.firstname = firstname
        self.middlename = middlename
        self.lastname = lastname
        self.snils = self._get_snils()
        self.document_id, self.document_series, \
            self.document_number = self._get_document(system, is_child=True)
        self.decl_id = get_id()
        self.decl_org_id = get_id()
        self.other_id = get_id()
        self.delegate_id = get_id()
        self.delegate_firstname = delegate_firstname
        self.delegate_middlename = delegate_middlename
        self.delegate_lastname = delegate_lastname
        self.delegate_snils = self._get_snils()
        self.delegate_doc_id, self.delegate_doc_series, \
            self.delegate_doc_number = self._get_document(system, is_child=False)
        self.delegate_person_id = get_id()
        self.organization = organization
        self.edu_id = get_id()
        self.project_id = get_id()
        self.org_activities_id = get_id()
        self.activities_id = get_id()
        self.achieve_id = get_id()
        self.ex_activities_id = get_id()
        self.self_stud_id = get_id()
        self.main_edu_class = get_id()
        self.basic_edu_mark_id = get_id()
        self.common_edu_mark_id = get_id()
        self.mid_edu_id = get_id()
        self.ann_mark_id = get_id()
        self.final_mark_id = get_id()
        self.employment_id = get_id()
        self.prof_edu_id = get_id()
        self.addit_edu_id = get_id()
        self.addit_edu_score_id = get_id()
        self.health_group = self._get_random_value(HealthGroups)  # HealthGroup
        self.long_term = self._get_random_value(LongTermTreatment)  # LongTermTreatment
        self.dis_group = self._get_random_value(DisabilityGroup)  # DisabilityGroup
        self.dis_date = '{year}-{month:02d}-{day:02d}'.format(
            year=randint(2016, 2017),
            month=randint(1, 12),
            day=randint(1, 28)
        )
        self.difficult_situation = self._get_random_value(DifficultSituation)  # DifficultSituation
        self.adapt_program = self._get_random_value(AdaptationProgram)  # AdaptationProgram
        self.phys_cult_group = self._get_random_value(PhysicalCultureGroup)  # PhysicalCultureGroup
        self.dob = self._get_date_of_birth(system, is_child=True)
        self.delegate_dob = self._get_date_of_birth(system, is_child=False)
        self.document_date = '{year}-{month:02d}-{day:02d}'.format(
            year=2015,
            month=randint(1, 12),
            day=randint(1, 28)
        )
        self.basic_edu_subject = self._get_subject(system)
        self.basic_score = randint(60, 100)
        self.basic_mark = self._get_mark(system, self.basic_score)
        self.common_edu_subject = self._get_subject(system)
        self.common_score = randint(60, 100)
        self.common_mark = self._get_mark(system, self.common_score)

    @staticmethod
    def _get_snils():
        numbers = [randint(0, 9) for _ in xrange(9)]
        str_num = [str(n) for n in numbers]
        multipls = list(xrange(1, 10))
        multipls.reverse()
        numbers = [numbers[i]*multipls[i] for i in xrange(len(numbers))]
        num_sum = sum(numbers)
        if num_sum == 100:
            check_sum = '00'
        else:
            check_sum = '%02d' % (num_sum % 101)

        for i in (3, 7):
            str_num.insert(i, '-')

        return ' '.join([''.join(str_num), check_sum])

    @staticmethod
    def _get_document(system, is_child=True):
        doc_id = get_id()
        SERIES_DICT = {
            1: u'V-ТФ',
            2: u'V-АГ'
        }
        if is_child and (system == 'dou' or system == 'school'):
            series = SERIES_DICT[randint(1, 2)]
        else:
            series = '%04d' % randint(0, 9999)
        number = '%06d'.encode('utf-8') % randint(0, 999999)
        return doc_id, series, number

    @staticmethod
    def _get_random_value(some_dict):
        keys = list(some_dict.iterkeys())
        keys_len = len(keys)
        return some_dict[keys[randint(0, keys_len-1)]]

    @staticmethod
    def _get_date_of_birth(system, is_child):
        if is_child:
            if system == 'dou':
                dob_year = randint(2014, 2015)
            elif system == 'school':
                dob_year = randint(2003, 2005)
            elif system == 'extedu':
                dob_year = randint(2005, 2015)
            else:
                dob_year = randint(1990, 2000)
        else:
            dob_year = randint(1970, 1980)

        return '{year}-{month:02d}-{day:02d}'.format(
            year=dob_year,
            month=randint(1, 12),
            day=randint(1, 28)
        )

    @staticmethod
    def _get_subject(system):
        if system == 'school':
            subj_list = SCHOOL_SUBJECTS
        else:
            subj_list = COLLEGE_SUBJECTS

        subj_len = len(subj_list)-1
        return subj_list[randint(0, subj_len)]

    @staticmethod
    def _get_mark(system, score):
        if system == 'college':
            return randint(3, 5)
        else:
            if 60 <= score < 75:
                mark = 3
            elif 75 <= score < 90:
                mark = 4
            else:
                mark = 5

        return mark


class Context(object):
    u"""
    Класс для передачи параметров запуска.
    """
    def __init__(self, persons=None, organizations=None, run_info=None):
        self.persons = persons
        self.organizations = organizations
        self.run_info = run_info